@@ -105,6 +105,19 @@ struct hnb_context *hnb_context_by_id(struct hnb_gw *gw, uint32_t cid)
 	return NULL;
 }
 
+struct hnb_context *hnb_context_by_identity_info(struct hnb_gw *gw, const char *identity_info)
+{
+	struct hnb_context *hnb;
+
+	llist_for_each_entry(hnb, &gw->hnb_list, list) {
+		if (strcmp(identity_info, hnb->identity_info) == 0)
+			return hnb;
+	}
+
+	return NULL;
+}
+
+
 unsigned hnb_contexts(const struct hnb_gw *gw)
 {
 	unsigned num_ctx = 0;
@@ -202,16 +215,6 @@ void ue_context_free(struct ue_context *ue)
 	llist_del(&ue->list);
 	talloc_free(ue);
 }
-static int hnb_close_cb(struct osmo_stream_srv *conn)
-{
-	struct hnb_context *hnb = osmo_stream_srv_get_data(conn);
-
-	/* This connection is about to be closed. Destroy the HNB context now. */
-	if (hnb)
-		hnb_context_release(hnb, false);
-
-	return 0;
-}
 
 static int hnb_read_cb(struct osmo_stream_srv *conn)
 {
@@ -234,10 +237,10 @@ static int hnb_read_cb(struct osmo_stream_srv *conn)
 	} else if (rc < 0) {
 		LOGP(DMAIN, LOGL_ERROR, "Error during sctp_recvmsg()\n");
 		/* FIXME: clean up after disappeared HNB */
-		hnb_context_release(hnb, true);
+		hnb_context_release(hnb);
 		goto out;
 	} else if (rc == 0) {
-		hnb_context_release(hnb, true);
+		hnb_context_release(hnb);
 		rc = -1;
 
 		goto out;
@@ -283,7 +286,7 @@ struct hnb_context *hnb_context_alloc(struct hnb_gw *gw, struct osmo_stream_srv_
 	INIT_LLIST_HEAD(&ctx->map_list);
 
 	ctx->gw = gw;
-	ctx->conn = osmo_stream_srv_create(tall_hnb_ctx, link, new_fd, hnb_read_cb, hnb_close_cb, ctx);
+	ctx->conn = osmo_stream_srv_create(tall_hnb_ctx, link, new_fd, hnb_read_cb, NULL, ctx);
 	if (!ctx->conn) {
 		LOGP(DMAIN, LOGL_INFO, "error while creating connection\n");
 		talloc_free(ctx);
@@ -294,7 +297,7 @@ struct hnb_context *hnb_context_alloc(struct hnb_gw *gw, struct osmo_stream_srv_
 	return ctx;
 }
 
-void hnb_context_release(struct hnb_context *ctx, bool destroy_conn)
+void hnb_context_release(struct hnb_context *ctx)
 {
 	struct hnbgw_context_map *map, *map2;
 
@@ -312,8 +315,7 @@ void hnb_context_release(struct hnb_context *ctx, bool destroy_conn)
 	}
 	ue_context_free_by_hnb(ctx->gw, ctx);
 
-	if (destroy_conn)
-		osmo_stream_srv_destroy(ctx->conn);
+	osmo_stream_srv_destroy(ctx->conn);
 
 	talloc_free(ctx);
 }
@@ -375,7 +377,7 @@ static struct {
 } hnbgw_cmdline_config = {
 	0,
 	"osmo-hnbgw.cfg",
-	true,
+	false,
 	false,
 	0,
 	NULL,
@@ -453,6 +455,11 @@ static void handle_options(int argc, char **argv)
 			break;
 		}
 	}
+
+	if (argc > optind) {
+		fprintf(stderr, "Unsupported positional arguments on command line\n");
+		exit(2);
+	}
 }
 
 CTRL_CMD_DEFINE_RO(hnb_info, "info");
@@ -525,18 +532,24 @@ int main(int argc, char **argv)
 	g_hnb_gw = hnb_gw_create(tall_hnb_ctx);
 	g_hnb_gw->config.rnc_id = 23;
 
-	rc = osmo_init_logging(&hnbgw_log_info);
+	rc = osmo_init_logging2(tall_hnb_ctx, &hnbgw_log_info);
 	if (rc < 0)
 		exit(1);
 
-	osmo_ss7_init();
+	rc = osmo_ss7_init();
+	if (rc < 0) {
+		LOGP(DMAIN, LOGL_FATAL, "osmo_ss7_init() failed with rc=%d\n", rc);
+		exit(1);
+	}
 
 	vty_info.copyright = osmo_hnbgw_copyright;
 	vty_init(&vty_info);
 
 	osmo_ss7_vty_init_asp(tall_hnb_ctx);
+	osmo_sccp_vty_init();
 	hnbgw_vty_init(g_hnb_gw, tall_hnb_ctx);
-	logging_vty_add_cmds(&hnbgw_log_info);
+	ctrl_vty_init(tall_hnb_ctx);
+	logging_vty_add_cmds();
 
 	/* Handle options after vty_init(), for --version */
 	handle_options(argc, argv);
