@@ -84,8 +84,11 @@ static int hnbgw_tx_hnb_register_rej(struct hnb_context *ctx)
 		osmo_stream_srv_set_flush_and_destroy(ctx->conn);
 	} else {
 		/* The message was not queued. Destroy the connection right away. */
-		hnb_context_release(ctx, true);
+		hnb_context_release(ctx);
+		return rc;
 	}
+
+	return 0;
 }
 
 static int hnbgw_tx_hnb_register_acc(struct hnb_context *ctx)
@@ -116,6 +119,8 @@ static int hnbgw_tx_hnb_register_acc(struct hnb_context *ctx)
 
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_HNBRegisterAccept, &accept_out);
 
+	LOGP(DHNBAP, LOGL_NOTICE, "Accepting HNB-REGISTER-REQ from %s\n", ctx->identity_info);
+
 	return hnbgw_hnbap_tx(ctx, msg);
 }
 
@@ -401,7 +406,7 @@ static int hnbgw_rx_hnb_deregister(struct hnb_context *ctx, ANY_t *in)
 		hnbap_cause_str(&ies.cause));
 
 	hnbap_free_hnbde_registeries(&ies);
-	hnb_context_release(ctx, true);
+	hnb_context_release(ctx);
 
 	return 0;
 }
