@@ -109,12 +109,30 @@ const struct value_string ranap_iu_event_type_names[] = {
 	{ 0, NULL }
 };
 
+static int global_iu_event(struct ranap_ue_conn_ctx *ue_ctx,
+			   enum ranap_iu_event_type type,
+			   void *data)
+{
+	if (!global_iu_event_cb)
+		return 0;
+
+	if (!ue_ctx->notification)
+		return 0;
+
+	return global_iu_event_cb(ue_ctx, type, data);
+}
+
 static struct ranap_ue_conn_ctx *ue_conn_ctx_alloc(struct ranap_iu_rnc *rnc, uint32_t conn_id)
 {
 	struct ranap_ue_conn_ctx *ctx = talloc_zero(talloc_iu_ctx, struct ranap_ue_conn_ctx);
 
 	ctx->rnc = rnc;
 	ctx->conn_id = conn_id;
+	ctx->notification = true;
+	ctx->free_on_release = false;
+	osmo_timer_setup(&ctx->release_timeout,
+			 (void *)(void *) ranap_iu_free_ue,
+			 ctx);
 	llist_add(&ctx->list, &ue_conn_ctx_list);
 
 	return ctx;
@@ -131,6 +149,17 @@ static struct ranap_ue_conn_ctx *ue_conn_ctx_find(uint32_t conn_id)
 	return NULL;
 }
 
+void ranap_iu_free_ue(struct ranap_ue_conn_ctx *ue_ctx)
+{
+	if (!ue_ctx)
+		return;
+
+	osmo_timer_del(&ue_ctx->release_timeout);
+	osmo_sccp_tx_disconn(g_scu, ue_ctx->conn_id, NULL, 0);
+	llist_del(&ue_ctx->list);
+	talloc_free(ue_ctx);
+}
+
 static struct ranap_iu_rnc *iu_rnc_alloc(uint16_t rnc_id, struct osmo_sccp_addr *addr)
 {
 	struct ranap_iu_rnc *rnc = talloc_zero(talloc_iu_ctx, struct ranap_iu_rnc);
@@ -277,9 +306,7 @@ int ranap_iu_tx_sec_mode_cmd(struct ranap_ue_conn_ctx *uectx, struct osmo_auth_v
 	osmo_prim_init(&prim->oph, SCCP_SAP_USER,
 			OSMO_SCU_PRIM_N_DATA,
 			PRIM_OP_REQUEST, msg);
-	osmo_sccp_user_sap_down(g_scu, &prim->oph);
-
-	return 0;
+	return osmo_sccp_user_sap_down(g_scu, &prim->oph);
 }
 
 int ranap_iu_tx_common_id(struct ranap_ue_conn_ctx *uectx, const char *imsi)
@@ -297,8 +324,7 @@ int ranap_iu_tx_common_id(struct ranap_ue_conn_ctx *uectx, const char *imsi)
 	osmo_prim_init(&prim->oph, SCCP_SAP_USER,
 			OSMO_SCU_PRIM_N_DATA,
 			PRIM_OP_REQUEST, msg);
-	osmo_sccp_user_sap_down(g_scu, &prim->oph);
-	return 0;
+	return osmo_sccp_user_sap_down(g_scu, &prim->oph);
 }
 
 static int iu_grnc_id_parse(struct iu_grnc_id *dst, struct RANAP_GlobalRNC_ID *src)
@@ -438,20 +464,19 @@ int ranap_iu_tx(struct msgb *msg_nas, uint8_t sapi)
 	osmo_prim_init(&prim->oph, SCCP_SAP_USER,
 			OSMO_SCU_PRIM_N_DATA,
 			PRIM_OP_REQUEST, msg);
-	osmo_sccp_user_sap_down(g_scu, &prim->oph);
-	return 0;
+	return osmo_sccp_user_sap_down(g_scu, &prim->oph);
 }
 
 /* Send Iu Release for the given UE connection.
- * If cause is NULL, the standard "No remaining RAB" cause is sent, otherwise
+ * If cause is NULL, Normal Release cause is sent, otherwise
  * the provided cause. */
 int ranap_iu_tx_release(struct ranap_ue_conn_ctx *ctx, const struct RANAP_Cause *cause)
 {
 	struct msgb *msg;
 	struct osmo_scu_prim *prim;
 	static const struct RANAP_Cause default_cause = {
-		.present = RANAP_Cause_PR_radioNetwork,
-		.choice.radioNetwork = RANAP_CauseRadioNetwork_no_remaining_rab,
+		.present = RANAP_Cause_PR_nAS,
+		.choice.radioNetwork = RANAP_CauseNAS_normal_release,
 	};
 
 	if (!cause)
@@ -467,6 +492,21 @@ int ranap_iu_tx_release(struct ranap_ue_conn_ctx *ctx, const struct RANAP_Cause
 	return osmo_sccp_user_sap_down(g_scu, &prim->oph);
 }
 
+void ranap_iu_tx_release_free(struct ranap_ue_conn_ctx *ctx,
+			     const struct RANAP_Cause *cause,
+			     int timeout)
+{
+	ctx->notification = false;
+	ctx->free_on_release = true;
+	int ret = ranap_iu_tx_release(ctx, cause);
+	if (ret) {
+		ranap_iu_free_ue(ctx);
+		return;
+	}
+
+	osmo_timer_schedule(&ctx->release_timeout, timeout, 0);
+}
+
 static int ranap_handle_co_iu_rel_req(struct ranap_ue_conn_ctx *ctx, RANAP_Iu_ReleaseRequestIEs_t *ies)
 {
 	LOGPIU(LOGL_INFO, "Received Iu Release Request, Sending Release Command\n");
@@ -491,7 +531,7 @@ static int ranap_handle_co_rab_ass_resp(struct ranap_ue_conn_ctx *ctx, RANAP_RAB
 			return rc;
 		}
 
-		rc = global_iu_event_cb(ctx, RANAP_IU_EVENT_RAB_ASSIGN, &setup_ies);
+		rc = global_iu_event(ctx, RANAP_IU_EVENT_RAB_ASSIGN, &setup_ies);
 
 		ranap_free_rab_setupormodifieditemies(&setup_ies);
 	}
@@ -557,11 +597,11 @@ static void cn_ranap_handle_co(void *ctx, ranap_message *message)
 		switch (message->procedureCode) {
 		case RANAP_ProcedureCode_id_SecurityModeControl:
 			/* Security Mode Complete */
-			rc = global_iu_event_cb(ctx, RANAP_IU_EVENT_SECURITY_MODE_COMPLETE, NULL);
+			rc = global_iu_event(ctx, RANAP_IU_EVENT_SECURITY_MODE_COMPLETE, NULL);
 			break;
 		case RANAP_ProcedureCode_id_Iu_Release:
 			/* Iu Release Complete */
-			rc = global_iu_event_cb(ctx, RANAP_IU_EVENT_IU_RELEASE, NULL);
+			rc = global_iu_event(ctx, RANAP_IU_EVENT_IU_RELEASE, NULL);
 			if (rc) {
 				LOGPIU(LOGL_ERROR, "Iu Release event: Iu Event callback returned %d\n",
 				       rc);
@@ -603,8 +643,24 @@ static void cn_ranap_handle_co(void *ctx, ranap_message *message)
 
 static int ranap_handle_cl_reset_req(void *ctx, RANAP_ResetIEs_t *ies)
 {
-	/* FIXME: send reset response */
-	return -1;
+	struct osmo_scu_prim *prim = (struct osmo_scu_prim *) ctx;
+	struct osmo_scu_unitdata_param *ud_prim = &prim->u.unitdata;
+	RANAP_GlobalRNC_ID_t *grnc_id = NULL;
+	struct msgb *resp;
+
+	OSMO_ASSERT(prim->oph.primitive == OSMO_SCU_PRIM_N_UNITDATA);
+
+	/* FIXME: verify ies.cN_DomainIndicator */
+
+	if (ies->presenceMask & RESETIES_RANAP_GLOBALRNC_ID_PRESENT)
+		grnc_id = &ies->globalRNC_ID;
+
+	/* send reset response */
+	resp = ranap_new_msg_reset_ack(ies->cN_DomainIndicator, grnc_id);
+	if (!resp)
+		return -ENOMEM;
+	resp->l2h = resp->data;
+	return osmo_sccp_tx_unitdata_msg(g_scu, &g_local_sccp_addr, &ud_prim->calling_addr, resp);
 }
 
 static int ranap_handle_cl_err_ind(void *ctx, RANAP_ErrorIndicationIEs_t *ies)
@@ -748,7 +804,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 	struct osmo_sccp_user *scu = _scu;
 	struct osmo_scu_prim *prim = (struct osmo_scu_prim *) oph;
 	struct osmo_prim_hdr *resp = NULL;
-	int rc;
+	int rc = -1;
 	struct ranap_ue_conn_ctx *ue;
 	struct new_ue_conn_ctx new_ctx = {};
 
@@ -757,7 +813,6 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 	switch (OSMO_PRIM_HDR(oph)) {
 	case OSMO_PRIM(OSMO_SCU_PRIM_N_CONNECT, PRIM_OP_CONFIRM):
 		/* confirmation of outbound connection */
-		rc = -1;
 		break;
 	case OSMO_PRIM(OSMO_SCU_PRIM_N_CONNECT, PRIM_OP_INDICATION):
 		/* indication of new inbound connection request*/
@@ -781,7 +836,26 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		LOGPIU(LOGL_DEBUG, "N-DISCONNECT.ind(%u)\n",
 		       prim->u.disconnect.conn_id);
 		ue = ue_conn_ctx_find(prim->u.disconnect.conn_id);
-		rc = ranap_cn_rx_co(cn_ranap_handle_co, ue, msgb_l2(oph->msg), msgb_l2len(oph->msg));
+		if (!ue)
+			break;
+
+		rc = 0;
+		if (msgb_l2len(oph->msg) > 0)
+			rc = ranap_cn_rx_co(cn_ranap_handle_co, ue, msgb_l2(oph->msg), msgb_l2len(oph->msg));
+
+		/* A Iu Release event might be used to free the UE in cn_ranap_handle_co. */
+		ue = ue_conn_ctx_find(prim->u.disconnect.conn_id);
+		if (!ue)
+			break;
+
+		global_iu_event(ue, RANAP_IU_EVENT_LINK_INVALIDATED, NULL);
+
+		/* A RANAP_IU_EVENT_LINK_INVALIDATED, can lead to a free */
+		ue = ue_conn_ctx_find(prim->u.disconnect.conn_id);
+		if (!ue)
+			break;
+		if (ue->free_on_release)
+			ranap_iu_free_ue(ue);
 		break;
 	case OSMO_PRIM(OSMO_SCU_PRIM_N_DATA, PRIM_OP_INDICATION):
 		/* connection-oriented data received */
@@ -789,16 +863,18 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)));
 		/* resolve UE context */
 		ue = ue_conn_ctx_find(prim->u.data.conn_id);
+		if (!ue)
+			break;
+
 		rc = ranap_cn_rx_co(cn_ranap_handle_co, ue, msgb_l2(oph->msg), msgb_l2len(oph->msg));
 		break;
 	case OSMO_PRIM(OSMO_SCU_PRIM_N_UNITDATA, PRIM_OP_INDICATION):
 		/* connection-less data received */
 		LOGPIU(LOGL_DEBUG, "N-UNITDATA.ind(%s)\n",
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)));
-		rc = ranap_cn_rx_cl(cn_ranap_handle_cl, scu, msgb_l2(oph->msg), msgb_l2len(oph->msg));
+		rc = ranap_cn_rx_cl(cn_ranap_handle_cl, prim, msgb_l2(oph->msg), msgb_l2len(oph->msg));
 		break;
 	default:
-		rc = -1;
 		break;
 	}
 
